==> SD35B_0.02.1_src.tar.gz: Sd35b/decoder/MessageCtrl.cpp <==
/*
 * MessageCtrl.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#include "stdafx.h"
#include "MessageCtrl.h"

#include "GraphicDriver.h"
#include "OffScreen.h"
#include "Kanakan.h"
#include "FileCtrl.h"

using namespace ALSD;
using namespace ALSD_MSGCTR;
using namespace ALSD_FILECTR;
/**********************************************************************
	RXgNV
**********************************************************************/
CMessageCtrl::CMessageCtrl( CScenarioDecoder *_lpScenarioDecoder )
{
	m_lpSD = _lpScenarioDecoder;

	m_bAutoPageChange = false;
	m_bHitAnyKey	= false;
	m_bClearFlag	= true;
	m_bHankaku		= false;
	m_bOutput		= true;
	m_bGetMessage	= false;
	m_NowWindow			 = 0;
	m_FontSize			= 16;
	m_FontColor			= 255;
	m_LineIncrement		= 2;
	m_CharIncrement		= 4;
	m_FontColor			= 255;
	m_BackColor			= 0;
	m_AnyKeyColor		= 255;
	m_FrameColor		= 255;
	m_MessageVarIndexNow	= 0;
	m_MessageVarIndexStart	= 0;
	m_bMessageVarUpdateR= true;
	m_MessageVarUpdateA = 0;

	m_SaveRectID	= -1;
	m_MessageAlpha	= 255;

	for( long i = 0; i < InfoMax; i++){
		m_lpMsgInfo[i].x = 100;
		m_lpMsgInfo[i].y = 300;
		m_lpMsgInfo[i].w = 400;
		m_lpMsgInfo[i].h =  90;
	}
}

CMessageCtrl::~CMessageCtrl()
{
}

/**********************************************************************
	WindowNA
**********************************************************************/
void CMessageCtrl::Clear( bool _bClearFlag )
{
	if( _bClearFlag ){
		if( m_lpMsgInfo[m_NowWindow].s && m_SaveRectID > -1){
			// ߂āA
			m_lpSD->GetGrpDriver()->RestoreRect( m_SaveRectID );
			m_SaveRectID = -1;
			// ۑ
			stGrpRect rect;
			rect.left = m_lpMsgInfo[m_NowWindow].x;
			rect.top  = m_lpMsgInfo[m_NowWindow].y;
			rect.right= rect.left + m_lpMsgInfo[m_NowWindow].w;
			rect.bottom=rect.top  + m_lpMsgInfo[m_NowWindow].h;
			m_SaveRectID = m_lpSD->GetGrpDriver()->SaveRect( &rect );
		
			// wFd˂
			m_lpSD->GetGrpDriver()->AddColorRect( 
				m_lpMsgInfo[m_NowWindow].x, m_lpMsgInfo[m_NowWindow].y,
				m_lpMsgInfo[m_NowWindow].w, m_lpMsgInfo[m_NowWindow].h,
				m_BackColor, m_MessageAlpha );
		
		}else{
			m_lpSD->GetGrpDriver()->FillRect( 
				m_lpMsgInfo[m_NowWindow].x, m_lpMsgInfo[m_NowWindow].y,
				m_lpMsgInfo[m_NowWindow].w, m_lpMsgInfo[m_NowWindow].h,
				m_BackColor );
		}
		// ĕ`̌Ăяo
		stGrpRect update;
		update.left = m_lpMsgInfo[m_NowWindow].x - 8;
		update.top	= m_lpMsgInfo[m_NowWindow].y - 8;
		update.right= update.left + m_lpMsgInfo[m_NowWindow].w + 16;
		update.bottom=update.top  + m_lpMsgInfo[m_NowWindow].h + 16;
		m_lpSD->GetGrpDriver()->OnDraw( &update );
	}

	//
	m_MsgPos.x = m_lpMsgInfo[m_NowWindow].x;
	m_MsgPos.y = m_lpMsgInfo[m_NowWindow].y + m_LineIncrement;
	m_bHitAnyKey = false;
}

/**********************************************************************
	gƂ̐ݒ
**********************************************************************/
void CMessageCtrl::SetMsgInfo( long _num, long _x, long _y,
							   long _w, long _h, long _s )
{
	m_lpMsgInfo[_num].x = _x;
	m_lpMsgInfo[_num].y = _y;
	m_lpMsgInfo[_num].w = _w;
	m_lpMsgInfo[_num].h = _h;
	m_lpMsgInfo[_num].s = _s;
}

void CMessageCtrl::ChangeMsg( long _num, long _w, long _c1, long c2,
							  long _n, long _m )
{
	// ȑO̕ۑf[^̂Ă
	if( m_SaveRectID > -1 ){
		m_lpSD->GetGrpDriver()->DeleteSaveRect( m_SaveRectID );
		m_SaveRectID = -1;
	}

	// ݒ
	m_NowWindow = _num;
	
	if( !_n )m_bClearFlag = true;
	else m_bClearFlag = false;

	m_FrameType = _w;

	// bZ[WWindow`
	switch( _w ){
		case ALSD_MSGWINDOW_FRAME_EMPTY:
			break;
		case ALSD_MSGWINDOW_FRAME_LINE:
			// ւ͂ȂKł
			// Z[uTCYg̓ɂȂĂ܂
			if( _m ){
				// A
				if( m_SaveRectID > -1 ){
					m_lpSD->GetGrpDriver()->RestoreRect( m_SaveRectID );
				}
			}else{
				// Ҕ
				// Window̕\̈ۑ
				if( m_lpMsgInfo[m_NowWindow].s ){
					stGrpRect rect;
					rect.left = m_lpMsgInfo[m_NowWindow].x;
					rect.top  = m_lpMsgInfo[m_NowWindow].y;
					rect.right= rect.left + m_lpMsgInfo[m_NowWindow].w;
					rect.bottom=rect.top  + m_lpMsgInfo[m_NowWindow].h;
					m_SaveRectID = m_lpSD->GetGrpDriver()->SaveRect( &rect );
				}
				// g\
				DrawLineFrame();
				// NA
				Clear(true);
			}
			break;
		case ALSD_MSGWINDOW_FRAME_CG:
			break;
	}
	//
	if( _m == 0 ){
	}else{
	}
	//
	m_MsgPos.x = m_lpMsgInfo[_num].x;
	m_MsgPos.y = m_lpMsgInfo[_num].y + m_LineIncrement;

}

/**********************************************************************
	Cg`
**********************************************************************/
void CMessageCtrl::DrawLineFrame( void )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();

	lpGD->StrokeRect(	m_lpMsgInfo[m_NowWindow].x - 1,  
						m_lpMsgInfo[m_NowWindow].y - 1,
						m_lpMsgInfo[m_NowWindow].w + 2,
						m_lpMsgInfo[m_NowWindow].h + 2,
						m_FrameColor );
	
	lpGD->StrokeRect(	m_lpMsgInfo[m_NowWindow].x - 3,  
						m_lpMsgInfo[m_NowWindow].y - 3,
						m_lpMsgInfo[m_NowWindow].w + 6,
						m_lpMsgInfo[m_NowWindow].h + 6,
						m_FrameColor );
	
	lpGD->StrokeRect(	m_lpMsgInfo[m_NowWindow].x - 4,  
						m_lpMsgInfo[m_NowWindow].y - 4,
						m_lpMsgInfo[m_NowWindow].w + 8,
						m_lpMsgInfo[m_NowWindow].h + 8,
						m_FrameColor );

	stGrpRect update;
	update.left = m_lpMsgInfo[m_NowWindow].x - 8;
	update.top	= m_lpMsgInfo[m_NowWindow].y - 8;
	update.right= update.left + m_lpMsgInfo[m_NowWindow].w + 16;
	update.bottom=update.top  + m_lpMsgInfo[m_NowWindow].h + 16;
	m_lpSD->GetGrpDriver()->OnDraw( &update );

}

/**********************************************************************
	bZ[W\
**********************************************************************/
void CMessageCtrl::Message( const char *_lpMsg )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();
	TRACE("--- Message ---\n" );

	//
	if( m_bHitAnyKey ){
		HitAnyKey();
		NextPage();
	}
	//
	string msg;
	if( m_bHankaku ){
		msg = kanakan::zentohan2( (const unsigned char *)_lpMsg );
	}else{
		msg = _lpMsg;
	}

	// bZ[W擾 
	if( m_bGetMessage ){
		char *lpStr = m_lpSD->GetExpDriver()->GetStrVariable( m_MessageVarIndexNow );
		strcat( lpStr, msg.c_str());
//		strcpy( lpStr, msg.c_str());
//		return;
	}

	// XV̈
	lpGD->SetFontSize( m_FontSize );
	long FontSize = m_lpSD->GetGrpDriver()->GetFontSize();
	stGrpRect update;
	update.left = m_MsgPos.x;
	update.top	= m_MsgPos.y;
	update.right= m_MsgPos.x + FontSize * strlen( msg.c_str() ) / 2;
	update.bottom=m_MsgPos.y + FontSize + m_LineIncrement;
	//
	if( m_bOutput ){
		DrawMessage( msg );
		lpGD->OnDraw( &update );
	}

	m_MsgPos.x = update.right;
}

void CMessageCtrl::DrawMessage( const string &_Msg )
{
	m_lpSD->GetGrpDriver()->DrawString( m_MsgPos.x, m_MsgPos.y, _Msg.c_str(), m_FontColor );
}


/**********************************************************************
	s
**********************************************************************/
void CMessageCtrl::NextLine( void )
{
	// bZ[W擾 
	if( m_bGetMessage && m_bMessageVarUpdateR ){
		m_MessageVarIndexNow++;
		return;
	}

	long FontSize = m_lpSD->GetGrpDriver()->GetFontSize();
	m_MsgPos.x = m_lpMsgInfo[m_NowWindow].x;
	m_MsgPos.y +=FontSize + m_LineIncrement;

	if( m_MsgPos.y + FontSize > m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h ){
		m_bHitAnyKey = true;
	}
}

/**********************************************************************
	y[W
**********************************************************************/
void CMessageCtrl::NextPage( void )
{
	Clear( m_bClearFlag );

	// bZ[W擾 
	if( m_bGetMessage ){
		switch( m_MessageVarUpdateA ){
			case 0:
				m_MessageVarIndexNow = m_MessageVarIndexStart;
				break;
			case 1:
				m_MessageVarIndexNow++;
				break;
			case 2:
				strcpy( m_lpSD->GetExpDriver()->GetStrVariable( m_MessageVarIndexNow ), "" );
				m_MessageVarIndexNow++;
				break;
			case 3:
			default:
				break;
		}
	}
}

/**********************************************************************
	\擾ϐ̍XV
**********************************************************************/
void CMessageCtrl::UpdateGetMessage( long _sw )
{
	switch( _sw ){
		case 0:
			m_MessageVarIndexNow--;break;
		case 1:
			m_MessageVarIndexNow++;break;
		case 2:
			if( strlen( m_lpSD->GetExpDriver()->GetStrVariable( m_MessageVarIndexNow )) ){
				m_MessageVarIndexNow++;
			}
			break;
	}
}

/**********************************************************************
	vvg
**********************************************************************/
void CMessageCtrl::DrawPrompt( void )
{
	m_lpSD->GetGrpDriver()->DrawString( 
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		lpPrompt, m_AnyKeyColor );
	m_lpSD->GetGrpDriver()->OnDraw( 			
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		m_FontSize + 1, m_FontSize + 1 );
}

void CMessageCtrl::ErasePrompt( void )
{
	m_lpSD->GetGrpDriver()->FillRect( 
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		m_FontSize + 1, m_FontSize + 1, m_BackColor );
	m_lpSD->GetGrpDriver()->OnDraw( 			
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		m_FontSize + 1, m_FontSize + 1 );
}

/**********************************************************************
	L[͑҂
**********************************************************************/
void CMessageCtrl::HitAnyKey( void )
{
	if( m_bAutoPageChange )return;

	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();
	bool bLButton = false;
	bool bKey	  = false;
	bool bPrompt  = true;
	long PromptCounter = 10;
	for(;;){
		// vvg\
		if( bPrompt ){
			DrawPrompt();
			if( !PromptCounter-- ){
				PromptCounter = 7;
				bPrompt = false;
			}
		}else{
			ErasePrompt();
			if( !PromptCounter-- ){
				PromptCounter = 10;
				bPrompt = true;
			}
		}

		// L[͎擾
		lpGD->Sleep( 50 );
		if( lpGD->GetKeyCode() ){
			bKey = true;
		}else{
			if( bKey )return;
		}
		if( lpGD->IsLButtonDown(NULL) ){
			bLButton = true;
		}else{
			if( bLButton )return;
		}
		if( lpGD->IsRButtonDown(NULL) )return;
	}

	ErasePrompt();
}

/**********************************************************************
	[hf[^Zbg
**********************************************************************/
void CMessageCtrl::SetLoadData( stSaveHeader *_lpSaveHeader )
{
	m_FontSize	= _lpSaveHeader->selFontSize;
	m_FontColor	= _lpSaveHeader->selFontColor;
	m_BackColor	= _lpSaveHeader->selClearColor;
	m_FrameColor= _lpSaveHeader->selFrameColor;

	for( long i = 0; i < InfoMax; i++){
		m_lpMsgInfo[i].x = _lpSaveHeader->msgWinInfo[i].x;
		m_lpMsgInfo[i].y = _lpSaveHeader->msgWinInfo[i].y;
		m_lpMsgInfo[i].w = _lpSaveHeader->msgWinInfo[i].w;
		m_lpMsgInfo[i].h = _lpSaveHeader->msgWinInfo[i].h;
	}
}

/**********************************************************************
	Z[uf[^ݒ肷
**********************************************************************/
void CMessageCtrl::GetSaveData( stSaveHeader *_lpSaveHeader )
{
	_lpSaveHeader->selFontSize	= m_FontSize;
	_lpSaveHeader->selFontColor	= m_FontColor;
	_lpSaveHeader->selClearColor= m_BackColor;
	_lpSaveHeader->selFrameColor= m_FrameColor;

	for( long i = 0; i < InfoMax; i++){
		_lpSaveHeader->msgWinInfo[i].x = m_lpMsgInfo[i].x;
		_lpSaveHeader->msgWinInfo[i].y = m_lpMsgInfo[i].y;
		_lpSaveHeader->msgWinInfo[i].w = m_lpMsgInfo[i].w;
		_lpSaveHeader->msgWinInfo[i].h = m_lpMsgInfo[i].h;
	}
}

==> SD35B_0.02.2_src.tar.gz: Sd35b/decoder/MessageCtrl.cpp <==
/*
 * MessageCtrl.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#include "stdafx.h"
#include "MessageCtrl.h"

#include "GraphicDriver.h"
#include "OffScreen.h"
#include "Kanakan.h"
#include "FileCtrl.h"

using namespace ALSD;
using namespace ALSD_MSGCTR;
using namespace ALSD_FILECTR;
/**********************************************************************
	RXgNV
**********************************************************************/
CMessageCtrl::CMessageCtrl( CScenarioDecoder *_lpScenarioDecoder )
{
	m_lpSD = _lpScenarioDecoder;

	m_bAutoPageChange = false;
	m_bAutoPageStop	  = true;
	m_bHitAnyKey	= false;
	m_bClearFlag	= true;
	m_bHankaku		= false;
	m_bOutput		= true;
	m_bGetMessage	= false;
	m_NowWindow			 = 0;
	m_FontSize			= 16;
	m_FontColor			= 255;
	m_LineIncrement		= 2;
	m_CharIncrement		= 4;
	m_FontColor			= 255;
	m_BackColor			= 0;
	m_AnyKeyColor		= 255;
	m_FrameColor		= 255;
	m_MessageVarIndexNow	= 0;
	m_MessageVarIndexStart	= 0;
	m_bMessageVarUpdateR= true;
	m_MessageVarUpdateA = 0;

	m_SaveRectID	= -1;
	m_MessageAlpha	= 255;

	for( long i = 0; i < InfoMax; i++){
		m_lpMsgInfo[i].x = 100;
		m_lpMsgInfo[i].y = 300;
		m_lpMsgInfo[i].w = 400;
		m_lpMsgInfo[i].h =  90;
	}
}

CMessageCtrl::~CMessageCtrl()
{
}

/**********************************************************************
	WindowNA
**********************************************************************/
void CMessageCtrl::Clear( bool _bClearFlag )
{
	if( _bClearFlag ){
		if( m_lpMsgInfo[m_NowWindow].s && m_SaveRectID > -1){
			// ߂āA
			m_lpSD->GetGrpDriver()->RestoreRect( m_SaveRectID );
			m_SaveRectID = -1;
			// ۑ
			stGrpRect rect;
			rect.left = m_lpMsgInfo[m_NowWindow].x;
			rect.top  = m_lpMsgInfo[m_NowWindow].y;
			rect.right= rect.left + m_lpMsgInfo[m_NowWindow].w;
			rect.bottom=rect.top  + m_lpMsgInfo[m_NowWindow].h;
			m_SaveRectID = m_lpSD->GetGrpDriver()->SaveRect( &rect );
		
			// wFd˂
			m_lpSD->GetGrpDriver()->AddColorRect( 
				m_lpMsgInfo[m_NowWindow].x, m_lpMsgInfo[m_NowWindow].y,
				m_lpMsgInfo[m_NowWindow].w, m_lpMsgInfo[m_NowWindow].h,
				m_BackColor, m_MessageAlpha );
		
		}else{
			m_lpSD->GetGrpDriver()->FillRect( 
				m_lpMsgInfo[m_NowWindow].x, m_lpMsgInfo[m_NowWindow].y,
				m_lpMsgInfo[m_NowWindow].w, m_lpMsgInfo[m_NowWindow].h,
				m_BackColor );
		}
		// ĕ`̌Ăяo
		stGrpRect update;
		update.left = m_lpMsgInfo[m_NowWindow].x;// - 8;
		update.top	= m_lpMsgInfo[m_NowWindow].y;// - 8;
		update.right= update.left + m_lpMsgInfo[m_NowWindow].w;// + 16;
		update.bottom=update.top  + m_lpMsgInfo[m_NowWindow].h;// + 16;
		m_lpSD->GetGrpDriver()->OnDraw( &update );
		if( m_FrameType ==  ALSD_MSGWINDOW_FRAME_LINE ){
			DrawLineFrame();
		}
	}

	//
	m_MsgPos.x = m_lpMsgInfo[m_NowWindow].x;
	m_MsgPos.y = m_lpMsgInfo[m_NowWindow].y + m_LineIncrement;
	m_bHitAnyKey = false;
}

/**********************************************************************
	gƂ̐ݒ
**********************************************************************/
void CMessageCtrl::SetMsgInfo( long _num, long _x, long _y,
							   long _w, long _h, long _s )
{
	m_lpMsgInfo[_num].x = _x;
	m_lpMsgInfo[_num].y = _y;
	m_lpMsgInfo[_num].w = _w;
	m_lpMsgInfo[_num].h = _h;
	m_lpMsgInfo[_num].s = _s;
}

void CMessageCtrl::ChangeMsg( long _num, long _w, long _c1, long c2,
							  long _n, long _m )
{
	// ȑO̕ۑf[^̂Ă
	if( m_SaveRectID > -1 && _m == 0 ){
		m_lpSD->GetGrpDriver()->DeleteSaveRect( m_SaveRectID );
		m_SaveRectID = -1;
	} 

	// ݒ
	m_NowWindow = _num;
	
	if( !_n )m_bClearFlag = true;
	else m_bClearFlag = false;

	m_FrameType = _w;

	if( _m ){
		// A
		if( m_SaveRectID > -1 ){
			m_lpSD->GetGrpDriver()->RestoreRect( m_SaveRectID );
			m_SaveRectID = -1;
		}
	}else{
		// Ҕ
		// Window̕\̈ۑ
		if( m_lpMsgInfo[m_NowWindow].s ){
			stGrpRect rect;
			rect.left = m_lpMsgInfo[m_NowWindow].x;
			rect.top  = m_lpMsgInfo[m_NowWindow].y;
			rect.right= rect.left + m_lpMsgInfo[m_NowWindow].w;
			rect.bottom=rect.top  + m_lpMsgInfo[m_NowWindow].h;
			m_SaveRectID = m_lpSD->GetGrpDriver()->SaveRect( &rect );
		}
	}

	// bZ[WWindow`
	switch( _w ){
		case ALSD_MSGWINDOW_FRAME_EMPTY:
			// NA
			Clear(m_bClearFlag);
			break;
		case ALSD_MSGWINDOW_FRAME_LINE:
			// g\
		//	DrawLineFrame();
			// NA
			Clear(m_bClearFlag);
			break;
		case ALSD_MSGWINDOW_FRAME_CG:
			Clear(m_bClearFlag);
			break;
	}

	//
	m_MsgPos.x = m_lpMsgInfo[_num].x;
	m_MsgPos.y = m_lpMsgInfo[_num].y + m_LineIncrement;

}

/**********************************************************************
	Cg`
**********************************************************************/
void CMessageCtrl::DrawLineFrame( void )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();

	lpGD->StrokeRect(	m_lpMsgInfo[m_NowWindow].x - 1,  
						m_lpMsgInfo[m_NowWindow].y - 1,
						m_lpMsgInfo[m_NowWindow].w + 2,
						m_lpMsgInfo[m_NowWindow].h + 2,
						m_FrameColor );
	
	lpGD->StrokeRect(	m_lpMsgInfo[m_NowWindow].x - 3,  
						m_lpMsgInfo[m_NowWindow].y - 3,
						m_lpMsgInfo[m_NowWindow].w + 6,
						m_lpMsgInfo[m_NowWindow].h + 6,
						m_FrameColor );
	
	lpGD->StrokeRect(	m_lpMsgInfo[m_NowWindow].x - 4,  
						m_lpMsgInfo[m_NowWindow].y - 4,
						m_lpMsgInfo[m_NowWindow].w + 8,
						m_lpMsgInfo[m_NowWindow].h + 8,
						m_FrameColor );

	stGrpRect update;
	update.left = m_lpMsgInfo[m_NowWindow].x - 8;
	update.top	= m_lpMsgInfo[m_NowWindow].y - 8;
	update.right= update.left + m_lpMsgInfo[m_NowWindow].w + 16;
	update.bottom=update.top  + m_lpMsgInfo[m_NowWindow].h + 16;
	m_lpSD->GetGrpDriver()->OnDraw( &update );

}

/**********************************************************************
	bZ[W\
**********************************************************************/
void CMessageCtrl::Message( const char *_lpMsg )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();
	TRACE("--- Message ---\n" );

	//
	if( m_bHitAnyKey ){
		HitAnyKey();
		NextPage();
	}
	//
	string msg;
	if( m_bHankaku ){
		msg = kanakan::zentohan2( (const unsigned char *)_lpMsg );
	}else{
		msg = _lpMsg;
	}

	// bZ[W擾 
	if( m_bGetMessage ){
		char *lpStr = m_lpSD->GetExpDriver()->GetStrVariable( m_MessageVarIndexNow );
		strcat( lpStr, msg.c_str());
//		strcpy( lpStr, msg.c_str());
//		return;
	}

	// XV̈
	lpGD->SetFontSize( m_FontSize );
	long FontSize = m_lpSD->GetGrpDriver()->GetFontSize();
	stGrpRect update;
	update.left = m_MsgPos.x;
	update.top	= m_MsgPos.y;
	update.right= m_MsgPos.x + FontSize * strlen( msg.c_str() ) / 2;
	update.bottom=m_MsgPos.y + FontSize + m_LineIncrement;
	//
	if( m_bOutput ){
		DrawMessage( msg );
		lpGD->OnDraw( &update );
	}

	m_MsgPos.x = update.right;
}

void CMessageCtrl::DrawMessage( const string &_Msg )
{
	m_lpSD->GetGrpDriver()->DrawString( m_MsgPos.x, m_MsgPos.y, _Msg.c_str(), m_FontColor );
}


/**********************************************************************
	s
**********************************************************************/
void CMessageCtrl::NextLine( void )
{
	// bZ[W擾 
	if( m_bGetMessage && m_bMessageVarUpdateR ){
		m_MessageVarIndexNow++;
		return;
	}

	long FontSize = m_lpSD->GetGrpDriver()->GetFontSize();
	m_MsgPos.x = m_lpMsgInfo[m_NowWindow].x;
	m_MsgPos.y +=FontSize + m_LineIncrement;

	if( m_MsgPos.y + FontSize > m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h ){
		m_bHitAnyKey = true;
	}
}

/**********************************************************************
	y[W
**********************************************************************/
void CMessageCtrl::NextPage( void )
{
	Clear( m_bClearFlag );

	// bZ[W擾 
	if( m_bGetMessage ){
		switch( m_MessageVarUpdateA ){
			case 0:
				m_MessageVarIndexNow = m_MessageVarIndexStart;
				break;
			case 1:
				m_MessageVarIndexNow++;
				break;
			case 2:
				strcpy( m_lpSD->GetExpDriver()->GetStrVariable( m_MessageVarIndexNow ), "" );
				m_MessageVarIndexNow++;
				break;
			case 3:
			default:
				break;
		}
	}
}

/**********************************************************************
	\擾ϐ̍XV
**********************************************************************/
void CMessageCtrl::UpdateGetMessage( long _sw )
{
	switch( _sw ){
		case 0:
			m_MessageVarIndexNow--;break;
		case 1:
			m_MessageVarIndexNow++;break;
		case 2:
			if( strlen( m_lpSD->GetExpDriver()->GetStrVariable( m_MessageVarIndexNow )) ){
				m_MessageVarIndexNow++;
			}
			break;
	}
}

/**********************************************************************
	vvg
**********************************************************************/
void CMessageCtrl::DrawPrompt( void )
{
	m_lpSD->GetGrpDriver()->DrawString( 
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		lpPrompt, m_AnyKeyColor );
	m_lpSD->GetGrpDriver()->OnDraw( 			
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		m_FontSize + 1, m_FontSize + 1 );
}

void CMessageCtrl::ErasePrompt( void )
{
	m_lpSD->GetGrpDriver()->FillRect( 
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		m_FontSize + 1, m_FontSize + 1, m_BackColor );
	m_lpSD->GetGrpDriver()->OnDraw( 			
		m_lpMsgInfo[m_NowWindow].x + m_lpMsgInfo[m_NowWindow].w - m_FontSize - 1,
		m_lpMsgInfo[m_NowWindow].y + m_lpMsgInfo[m_NowWindow].h - m_FontSize - 1,
		m_FontSize + 1, m_FontSize + 1 );
}

/**********************************************************************
	L[͑҂
**********************************************************************/
void CMessageCtrl::HitAnyKey( void )
{
	CGraphicDriver *lpGD = m_lpSD->GetGrpDriver();

	if( m_bAutoPageChange ){
		lpGD->Sleep( 20 );	
		return;
	}

	bool bLButton = false;
	bool bKey	  = false;
	bool bPrompt  = true;
	long PromptCounter = 10;
	for(;;){
		// vvg\
		if( bPrompt ){
			DrawPrompt();
			if( !PromptCounter-- ){
				PromptCounter = 7;
				bPrompt = false;
			}
		}else{
			ErasePrompt();
			if( !PromptCounter-- ){
				PromptCounter = 10;
				bPrompt = true;
			}
		}

		// L[͎擾
		lpGD->Sleep( 50 );
		if( lpGD->GetKeyCode() ){
			bKey = true;
		}else{
			if( bKey )return;
		}
		if( lpGD->IsLButtonDown(NULL) ){
			bLButton = true;
		}else{
			if( bLButton )return;
		}
		if( lpGD->IsRButtonDown(NULL) )return;
	}

	ErasePrompt();
}

/**********************************************************************
	[hf[^Zbg
**********************************************************************/
void CMessageCtrl::SetLoadData( stSaveHeader *_lpSaveHeader )
{
	m_FontSize	= _lpSaveHeader->msgFontSize;
	m_FontColor	= _lpSaveHeader->msgFontColor;
	m_BackColor	= _lpSaveHeader->msgClearColor;
	m_FrameColor= _lpSaveHeader->msgFrameColor;

	for( long i = 0; i < InfoMax; i++){
		m_lpMsgInfo[i].x = _lpSaveHeader->msgWinInfo[i].x;
		m_lpMsgInfo[i].y = _lpSaveHeader->msgWinInfo[i].y;
		m_lpMsgInfo[i].w = _lpSaveHeader->msgWinInfo[i].w;
		m_lpMsgInfo[i].h = _lpSaveHeader->msgWinInfo[i].h;
	}
}

/**********************************************************************
	Z[uf[^ݒ肷
**********************************************************************/
void CMessageCtrl::GetSaveData( stSaveHeader *_lpSaveHeader )
{
	_lpSaveHeader->msgFontSize	= m_FontSize;
	_lpSaveHeader->msgFontColor	= m_FontColor;
	_lpSaveHeader->msgClearColor= m_BackColor;
	_lpSaveHeader->msgFrameColor= m_FrameColor;

	for( long i = 0; i < InfoMax; i++){
		_lpSaveHeader->msgWinInfo[i].x = m_lpMsgInfo[i].x;
		_lpSaveHeader->msgWinInfo[i].y = m_lpMsgInfo[i].y;
		_lpSaveHeader->msgWinInfo[i].w = m_lpMsgInfo[i].w;
		_lpSaveHeader->msgWinInfo[i].h = m_lpMsgInfo[i].h;
	}
}
